@@ -149,7 +149,7 @@ webkit_pref_callback_user_stylesheet (GSettings  *settings,
 
   user_style_sheet_monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &error);
   if (!user_style_sheet_monitor) {
-    g_warning ("Could not create a file monitor for %s: %s\n", g_file_get_uri (file), error->message);
+    g_warning ("Could not create a file monitor for %s: %s", g_file_get_uri (file), error->message);
     g_error_free (error);
   } else {
     g_signal_connect (user_style_sheet_monitor, "changed", G_CALLBACK (user_style_sheet_file_changed), NULL);
@@ -256,7 +256,7 @@ webkit_pref_callback_user_javascript (GSettings  *settings,
 
   user_javascript_monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &error);
   if (!user_javascript_monitor)
-    g_warning ("Could not create a file monitor for %s: %s\n", g_file_get_uri (file), error->message);
+    g_warning ("Could not create a file monitor for %s: %s", g_file_get_uri (file), error->message);
   else
     g_signal_connect (user_javascript_monitor, "changed", G_CALLBACK (user_javascript_file_changed), NULL);
 }
@@ -452,8 +452,8 @@ webkit_pref_callback_enable_website_data_storage (GSettings  *settings,
                                                   const char *key,
                                                   gpointer    data)
 {
-  EphyEmbedShell *shell;
-  WebKitWebContext *context;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+  WebKitNetworkSession *network_session;
   WebKitCookieManager *manager;
   gboolean value;
 
@@ -461,9 +461,8 @@ webkit_pref_callback_enable_website_data_storage (GSettings  *settings,
   webkit_settings_set_enable_html5_database (webkit_settings, value);
   webkit_settings_set_enable_html5_local_storage (webkit_settings, value);
 
-  shell = ephy_embed_shell_get_default ();
-  context = ephy_embed_shell_get_web_context (shell);
-  manager = webkit_web_context_get_cookie_manager (context);
+  network_session = ephy_embed_shell_get_network_session (shell);
+  manager = webkit_network_session_get_cookie_manager (network_session);
   webkit_cookie_manager_set_accept_policy (manager, value ? WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS : WEBKIT_COOKIE_POLICY_ACCEPT_NEVER);
 }
 
@@ -476,84 +475,108 @@ webkit_pref_callback_hardware_acceleration_policy (GSettings  *settings,
 
   value = g_settings_get_enum (settings, key);
   switch (value) {
-    case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_ALWAYS:
-      webkit_settings_set_hardware_acceleration_policy (webkit_settings,
-                                                        WEBKIT_HARDWARE_ACCELERATION_POLICY_ALWAYS);
-      break;
     case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_NEVER:
       webkit_settings_set_hardware_acceleration_policy (webkit_settings,
                                                         WEBKIT_HARDWARE_ACCELERATION_POLICY_NEVER);
       break;
-    case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_ON_DEMAND:
+    case EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY_ALWAYS:
     /* Fallthrough */
     default:
       webkit_settings_set_hardware_acceleration_policy (webkit_settings,
-                                                        WEBKIT_HARDWARE_ACCELERATION_POLICY_ON_DEMAND);
+                                                        WEBKIT_HARDWARE_ACCELERATION_POLICY_ALWAYS);
       break;
   }
 }
 
 static const PrefData webkit_pref_entries[] = {
   /* Epiphany font settings */
-  { EPHY_PREFS_WEB_SCHEMA,
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_SERIF_FONT,
     "default-font-size",
-    webkit_pref_callback_font_size },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_font_size
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_MONOSPACE_FONT,
     "default-monospace-font-size",
-    webkit_pref_callback_font_size },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_font_size
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_SERIF_FONT,
     "default-font-family",
-    webkit_pref_callback_font_family },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_font_family
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_SANS_SERIF_FONT,
     "sans-serif-font-family",
-    webkit_pref_callback_font_family },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_font_family
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_MONOSPACE_FONT,
     "monospace-font-family",
-    webkit_pref_callback_font_family },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_font_family
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_SERIF_FONT,
     "serif-font-family",
-    webkit_pref_callback_font_family },
+    webkit_pref_callback_font_family
+  },
 
-  { EPHY_PREFS_WEB_SCHEMA,
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_USE_GNOME_FONTS,
     NULL,
-    webkit_pref_callback_gnome_fonts },
+    webkit_pref_callback_gnome_fonts
+  },
 
-  { EPHY_PREFS_WEB_SCHEMA,
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
     NULL,
-    webkit_pref_callback_enable_spell_checking },
+    webkit_pref_callback_enable_spell_checking
+  },
 
-  { EPHY_PREFS_WEB_SCHEMA,
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_ENABLE_USER_CSS,
     "user-stylesheet-uri",
-    webkit_pref_callback_user_stylesheet },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_user_stylesheet
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_ENABLE_USER_JS,
     "user-javascript-uri",
-    webkit_pref_callback_user_javascript },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_user_javascript
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_LANGUAGE,
     "accept-language",
-    webkit_pref_callback_accept_languages },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_accept_languages
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_USER_AGENT,
     "user-agent",
-    webkit_pref_callback_user_agent },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_user_agent
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_ENABLE_WEBSITE_DATA_STORAGE,
     NULL,
-    webkit_pref_callback_enable_website_data_storage },
-  { EPHY_PREFS_WEB_SCHEMA,
+    webkit_pref_callback_enable_website_data_storage
+  },
+  {
+    EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY,
     "hardware-acceleration-policy",
-    webkit_pref_callback_hardware_acceleration_policy },
+    webkit_pref_callback_hardware_acceleration_policy
+  },
 };
 
 static gpointer
@@ -561,10 +584,8 @@ ephy_embed_prefs_init (gpointer user_data)
 {
   guint i;
 
-  webkit_settings = webkit_settings_new_with_settings ("enable-back-forward-navigation-gestures", TRUE,
-                                                       "enable-developer-extras", TRUE,
+  webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
                                                        "enable-dns-prefetching", TRUE,
-                                                       "enable-plugins", FALSE,
                                                        "enable-media-stream", TRUE,
                                                        "enable-smooth-scrolling", TRUE,
                                                        "javascript-can-open-windows-automatically", TRUE,
@@ -603,6 +624,11 @@ ephy_embed_prefs_init (gpointer user_data)
                    EPHY_PREFS_WEB_ENABLE_SITE_SPECIFIC_QUIRKS,
                    webkit_settings, "enable-site-specific-quirks",
                    G_SETTINGS_BIND_GET);
+  g_settings_bind (EPHY_SETTINGS_WEB,
+                   EPHY_PREFS_WEB_ENABLE_NAVIGATION_GESTURES,
+                   webkit_settings, "enable-back-forward-navigation-gestures",
+                   G_SETTINGS_BIND_GET);
+
   return webkit_settings;
 }
 
