@@ -23,7 +23,6 @@
 #include "config.h"
 #include "ephy-header-bar.h"
 
-#include "ephy-add-bookmark-popover.h"
 #include "ephy-desktop-utils.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
@@ -57,8 +56,6 @@ struct _EphyHeaderBar {
   GtkWidget *header_bar;
   EphyWindow *window;
   EphyTitleWidget *title_widget;
-  GtkRevealer *start_revealer;
-  GtkRevealer *end_revealer;
   EphyActionBarStart *action_bar_start;
   EphyActionBarEnd *action_bar_end;
   GtkWidget *page_menu_button;
@@ -70,7 +67,7 @@ struct _EphyHeaderBar {
   guint popover_hide_timeout_id;
 };
 
-G_DEFINE_TYPE (EphyHeaderBar, ephy_header_bar, ADW_TYPE_BIN)
+G_DEFINE_FINAL_TYPE (EphyHeaderBar, ephy_header_bar, ADW_TYPE_BIN)
 
 static void
 ephy_header_bar_set_property (GObject      *object,
@@ -121,14 +118,12 @@ sync_chromes_visibility (EphyHeaderBar *header_bar)
   gtk_widget_set_visible (header_bar->page_menu_button, chrome & EPHY_WINDOW_CHROME_MENU);
 }
 
-static gboolean
+static void
 hide_timeout_cb (EphyHeaderBar *header_bar)
 {
   gtk_menu_button_popdown (GTK_MENU_BUTTON (header_bar->page_menu_button));
 
   header_bar->popover_hide_timeout_id = 0;
-
-  return G_SOURCE_REMOVE;
 }
 
 void
@@ -138,25 +133,18 @@ fullscreen_changed_cb (EphyHeaderBar *header_bar)
 
   g_object_get (header_bar->window, "fullscreened", &fullscreen, NULL);
 
-  gtk_header_bar_set_show_title_buttons (GTK_HEADER_BAR (header_bar->header_bar), !fullscreen);
+  adw_header_bar_set_show_start_title_buttons (ADW_HEADER_BAR (header_bar->header_bar), !fullscreen);
+  adw_header_bar_set_show_end_title_buttons (ADW_HEADER_BAR (header_bar->header_bar), !fullscreen);
   gtk_widget_set_visible (header_bar->restore_button, fullscreen);
 
   if (fullscreen) {
     g_clear_handle_id (&header_bar->popover_hide_timeout_id, g_source_remove);
 
     header_bar->popover_hide_timeout_id =
-      g_timeout_add (POPOVER_HIDE_DELAY, (GSourceFunc)hide_timeout_cb, header_bar);
+      g_timeout_add_once (POPOVER_HIDE_DELAY, (GSourceOnceFunc)hide_timeout_cb, header_bar);
   }
 }
 
-static void
-update_revealer_visibility (GtkRevealer *revealer)
-{
-  gtk_widget_set_visible (GTK_WIDGET (revealer),
-                          gtk_revealer_get_reveal_child (revealer) ||
-                          gtk_revealer_get_child_revealed (revealer));
-}
-
 static gboolean
 remove_menu_item (GMenu      *menu,
                   const char *action_name)
@@ -229,21 +217,13 @@ ephy_header_bar_constructed (GObject *object)
                            G_CONNECT_SWAPPED);
 
   /* Header bar */
-  header_bar->header_bar = gtk_header_bar_new ();
+  header_bar->header_bar = adw_header_bar_new ();
   adw_bin_set_child (ADW_BIN (header_bar), header_bar->header_bar);
 
   /* Start action elements */
   header_bar->action_bar_start = ephy_action_bar_start_new ();
-  header_bar->start_revealer = GTK_REVEALER (gtk_revealer_new ());
-  g_signal_connect (header_bar->start_revealer, "notify::child-revealed",
-                    G_CALLBACK (update_revealer_visibility), NULL);
-  g_signal_connect (header_bar->start_revealer, "notify::reveal-child",
-                    G_CALLBACK (update_revealer_visibility), NULL);
-  gtk_revealer_set_transition_type (GTK_REVEALER (header_bar->start_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_RIGHT);
-  gtk_revealer_set_child (GTK_REVEALER (header_bar->start_revealer), GTK_WIDGET (header_bar->action_bar_start));
-
-  gtk_header_bar_pack_start (GTK_HEADER_BAR (header_bar->header_bar),
-                             GTK_WIDGET (header_bar->start_revealer));
+  adw_header_bar_pack_start (ADW_HEADER_BAR (header_bar->header_bar),
+                             GTK_WIDGET (header_bar->action_bar_start));
 
   embed_shell = ephy_embed_shell_get_default ();
 
@@ -254,7 +234,7 @@ ephy_header_bar_constructed (GObject *object)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_location_entry_new ());
 
   event_box = adw_bin_new ();
-  gtk_header_bar_set_title_widget (GTK_HEADER_BAR (header_bar->header_bar), event_box);
+  adw_header_bar_set_title_widget (ADW_HEADER_BAR (header_bar->header_bar), event_box);
   gtk_widget_set_name (event_box, "title-box-container");
 
   if (is_desktop_pantheon ()) {
@@ -276,19 +256,12 @@ ephy_header_bar_constructed (GObject *object)
     adw_bin_set_child (ADW_BIN (event_box), clamp);
   }
 
-  if (EPHY_IS_LOCATION_ENTRY (header_bar->title_widget)) {
-    EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (header_bar->title_widget);
-    GtkWidget *popover = ephy_add_bookmark_popover_new ();
-
-    ephy_location_entry_set_add_bookmark_popover (lentry, GTK_POPOVER (popover));
-  }
-
   /* Fullscreen restore button */
   header_bar->restore_button = gtk_button_new_from_icon_name ("view-restore-symbolic");
   gtk_widget_set_visible (header_bar->restore_button, FALSE);
   gtk_actionable_set_action_name (GTK_ACTIONABLE (header_bar->restore_button),
                                   "win.fullscreen");
-  gtk_header_bar_pack_end (GTK_HEADER_BAR (header_bar->header_bar),
+  adw_header_bar_pack_end (ADW_HEADER_BAR (header_bar->header_bar),
                            GTK_WIDGET (header_bar->restore_button));
 
   /* Page Menu */
@@ -302,7 +275,6 @@ ephy_header_bar_constructed (GObject *object)
   header_bar->zoom_level_label = GTK_WIDGET (gtk_builder_get_object (builder, "zoom-level"));
 
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    remove_menu_item (menu, "app.new-window");
     remove_menu_item (menu, "app.new-incognito");
     remove_menu_item (menu, "app.reopen-closed-tab");
     remove_menu_item (menu, "win.save-as-application");
@@ -312,6 +284,7 @@ ephy_header_bar_constructed (GObject *object)
     remove_menu_item (menu, "app.help");
     remove_menu_item (menu, "app.firefox-sync-dialog");
     remove_menu_item (menu, "import-export");
+    remove_menu_item (menu, "webapps");
   } else if (ephy_is_running_inside_sandbox ()) {
     remove_menu_item (menu, "app.run-in-background");
 
@@ -345,20 +318,12 @@ ephy_header_bar_constructed (GObject *object)
   gtk_menu_button_set_popover (GTK_MENU_BUTTON (button), header_bar->page_menu_popover);
   g_object_unref (builder);
 
-  gtk_header_bar_pack_end (GTK_HEADER_BAR (header_bar->header_bar), button);
+  adw_header_bar_pack_end (ADW_HEADER_BAR (header_bar->header_bar), button);
 
   /* End action elements */
   header_bar->action_bar_end = ephy_action_bar_end_new ();
-  header_bar->end_revealer = GTK_REVEALER (gtk_revealer_new ());
-  g_signal_connect (header_bar->end_revealer, "notify::child-revealed",
-                    G_CALLBACK (update_revealer_visibility), NULL);
-  g_signal_connect (header_bar->end_revealer, "notify::reveal-child",
-                    G_CALLBACK (update_revealer_visibility), NULL);
-  gtk_revealer_set_transition_type (GTK_REVEALER (header_bar->end_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_LEFT);
-  gtk_revealer_set_child (GTK_REVEALER (header_bar->end_revealer), GTK_WIDGET (header_bar->action_bar_end));
-
-  gtk_header_bar_pack_end (GTK_HEADER_BAR (header_bar->header_bar),
-                           GTK_WIDGET (header_bar->end_revealer));
+  adw_header_bar_pack_end (ADW_HEADER_BAR (header_bar->header_bar),
+                           GTK_WIDGET (header_bar->action_bar_end));
 
   /* Sync the size of placeholder in EphyActionBarStart with downloads button */
   downloads_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -455,19 +420,16 @@ void
 ephy_header_bar_set_adaptive_mode (EphyHeaderBar    *header_bar,
                                    EphyAdaptiveMode  adaptive_mode)
 {
-  ephy_action_bar_end_set_show_bookmark_button (header_bar->action_bar_end,
-                                                adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW);
-
   switch (adaptive_mode) {
     case EPHY_ADAPTIVE_MODE_NORMAL:
-      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), TRUE);
-      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), TRUE);
+      gtk_widget_set_visible (GTK_WIDGET (header_bar->action_bar_start), TRUE);
+      gtk_widget_set_visible (GTK_WIDGET (header_bar->action_bar_end), TRUE);
       gtk_widget_set_visible (header_bar->combined_stop_reload_button, FALSE);
 
       break;
     case EPHY_ADAPTIVE_MODE_NARROW:
-      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), FALSE);
-      gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), FALSE);
+      gtk_widget_set_visible (GTK_WIDGET (header_bar->action_bar_start), FALSE);
+      gtk_widget_set_visible (GTK_WIDGET (header_bar->action_bar_end), FALSE);
       gtk_widget_set_visible (header_bar->combined_stop_reload_button, TRUE);
 
       break;
