@@ -158,7 +158,7 @@ entry_activate_cb (EphyLocationEntry      *entry,
     /* After normalization there are still some cases that are
      * impossible to tell apart. One example is <URI>:<PORT> and <NON
      * WEB SCHEME>:<DATA>. To fix this, let's do a HEAD request to the
-     * effective URI prefxed with http://; if we get OK Status the URI
+     * effective URI prefixed with http://; if we get OK Status the URI
      * exists, and we'll go ahead, otherwise we'll try to launch a
      * proper handler through gtk_show_uri. We only do this in
      * ephy_web_view_load_url, since this case is only relevant for URIs
@@ -211,6 +211,9 @@ get_location_cb (EphyLocationEntry      *entry,
   const char *address;
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (controller->window));
+  if (!embed)
+    return NULL;
+
   address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
   return ephy_embed_utils_is_no_show_address (address) ? NULL : ephy_uri_decode (address);
